#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Ce module permet de chiffrer des chaines de caractères et des fichier.
Il permet une grande sécurité, et rapidité pour les conversations instantanées.
'''

__all__ = ["NCrypter", "NExchange"]
from NimingCypher.Niming import NCrypter
from NimingCypher.Exchange import NExchange
